/*
 * World Storage API
 *
 * API ensuring interoperability between an authoring tool and a World Storage service
 *
 * The version of the OpenAPI document: 0.0.4
 * 
 * Generated by: https://openapi-generator.tech
 */

using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Http;
using Swashbuckle.AspNetCore.Annotations;
using Swashbuckle.AspNetCore.SwaggerGen;
using Newtonsoft.Json;
using Org.OpenAPITools.Attributes;
using Org.OpenAPITools.Models;
using Org.OpenAPITools.Services;
using MongoDB.Driver;

namespace Org.OpenAPITools.Controllers
{
  /// <summary>
  /// 
  /// </summary>
  [ApiController]
  public class WorldLinksApiControllerImpl : WorldLinksApiController
  {

    private readonly WorldLinkService _worldLinkService;

    /// <summary>
    /// 
    /// </summary>
    public WorldLinksApiControllerImpl(WorldLinkService worldLinkService)
    {
      _worldLinkService = worldLinkService;
    }

    /// <summary>
    /// Create a link between world anchors and trackables.
    /// </summary>
    /// <param name="worldLink">The link to be added to the world storage.</param>
    /// <response code="200">OK, returns the UUID of the link defined by the world storage.</response>
    /// <response code="201">Null response</response>
    /// <response code="400">Bad request.</response>
    /// <response code="409">Invalid UUID, id must be a Nil value.</response>
    /// <response code="0">Unexpected error.</response>
    [HttpPost]
    [Route("/worldLinks")]
    [Consumes("application/json")]
    [ValidateModelState]
    [SwaggerOperation("AddWorldLink")]
    [SwaggerResponse(statusCode: 200, type: typeof(string), description: "OK, returns the UUID of the link defined by the world storage.")]
    [SwaggerResponse(statusCode: 0, type: typeof(Error), description: "Unexpected error.")]
    public override IActionResult AddWorldLink([FromBody] WorldLink worldLink)
    {
      if (String.IsNullOrEmpty(worldLink.UUID.ToString()))
      {
        worldLink.UUID = Guid.NewGuid();
      }
      try
      {
        WorldLink myworldlink = _worldLinkService.Create(worldLink);
        return new ObjectResult(myworldlink);
      }
      catch (Exception e)
      {
        return StatusCode(400, e.Message);
      }
    }

    /// <summary>
    /// Deletes a worldLink.
    /// </summary>
    /// <param name="worldLinkUUID">link id to delete</param>
    /// <response code="200">OK</response>
    /// <response code="400">Invalid UUID supplied.</response>
    /// <response code="404">Not found, could not find UUID in database.</response>
    [HttpDelete]
    [Route("/worldLinks/{worldLinkUUID}")]
    [ValidateModelState]
    [SwaggerOperation("DeleteWorldLink")]
    public override IActionResult DeleteWorldLink([FromRoute(Name = "worldLinkUUID")][Required] Guid worldLinkUUID)
    {
      DeleteResult answer = _worldLinkService.Remove((worldLinkUUID));
      return (answer.IsAcknowledged && answer.DeletedCount > 0) ? new ObjectResult("ok") : StatusCode(404, "Not found, could not find UUID in database.");
    }

    /// <summary>
    /// Retrieve end objects connected to this world link by its UUID.
    /// </summary>
    /// <param name="worldLinkUUID">UUID of the link from which you will retrieve the end objects.</param>
    /// <response code="200">Successful operation.</response>
    /// <response code="400">Invalid UUID supplied.</response>
    /// <response code="404">Not found, could not find UUID in database.</response>
    [HttpGet]
    [Route("/worldLinks/attached/{worldLinkUUID}")]
    [ValidateModelState]
    [SwaggerOperation("GetAttachedObjectsFromUUID")]
    [SwaggerResponse(statusCode: 200, type: typeof(AttachedObjects), description: "Successful operation.")]
    public override IActionResult GetAttachedObjectsFromUUID([FromRoute(Name = "worldLinkUUID")][Required] Guid worldLinkUUID)
    {
      WorldLink myworldlink = _worldLinkService.Get(worldLinkUUID);
      if (null == myworldlink)
      {
        return StatusCode(404, "not found, could not find UUID in database");
      }
      AttachedObjects attachedobjects = new AttachedObjects();
      attachedobjects.UUID = myworldlink.UUID;
      attachedobjects.UUIDFrom = myworldlink.UUIDFrom;
      attachedobjects.UUIDTo = myworldlink.UUIDTo;
      if (null != _worldLinkService.Get(attachedobjects.UUIDFrom)) attachedobjects.TypeFrom = ObjectType.WorldLinkEnum;
      if (null != _worldLinkService.GetAnchor(attachedobjects.UUIDFrom)) attachedobjects.TypeFrom = ObjectType.WorldAnchorEnum;
      if (null != _worldLinkService.GetTrackable(attachedobjects.UUIDFrom)) attachedobjects.TypeFrom = ObjectType.TrackableEnum;
      if (null != _worldLinkService.Get(attachedobjects.UUIDTo)) attachedobjects.TypeTo = ObjectType.WorldLinkEnum;
      if (null != _worldLinkService.GetAnchor(attachedobjects.UUIDTo)) attachedobjects.TypeTo = ObjectType.WorldAnchorEnum;
      if (null != _worldLinkService.GetTrackable(attachedobjects.UUIDTo)) attachedobjects.TypeTo = ObjectType.TrackableEnum;

      return new ObjectResult(attachedobjects);
    }

    /// <summary>
    /// Find a link by its UUID.
    /// </summary>
    /// <param name="worldLinkUUID">UUID of the link to retrieve.</param>
    /// <response code="200">Successful operation.</response>
    /// <response code="400">Invalid UUID supplied.</response>
    /// <response code="404">Not found, could not find UUID in database.</response>
    [HttpGet]
    [Route("/worldLinks/{worldLinkUUID}")]
    [ValidateModelState]
    [SwaggerOperation("GetWorldLinkById")]
    [SwaggerResponse(statusCode: 200, type: typeof(WorldLink), description: "Successful operation.")]
    public override IActionResult GetWorldLinkById([FromRoute(Name = "worldLinkUUID")][Required] Guid worldLinkUUID)
    {
      WorldLink myworldlink = _worldLinkService.Get(worldLinkUUID);
      return (null != myworldlink) ? new ObjectResult(myworldlink) : StatusCode(404, "Not found, could not find UUID in database.");
    }

    /// <summary>
    /// Returns the list of all links defined by the world storage.
    /// </summary>
    /// <response code="200">OK returns all the worldLinks defined by the world storage.</response>
    /// <response code="201">Null response</response>
    /// <response code="0">Unexpected error.</response>
    [HttpGet]
    [Route("/worldLinks")]
    [ValidateModelState]
    [SwaggerOperation("GetWorldLinks")]
    [SwaggerResponse(statusCode: 200, type: typeof(List<WorldLink>), description: "OK returns all the worldLinks defined by the world storage.")]
    [SwaggerResponse(statusCode: 0, type: typeof(Error), description: "Unexpected error.")]
    public override IActionResult GetWorldLinks()
    {
      List<WorldLink> worldlinklist = _worldLinkService.Get();
      return new ObjectResult(worldlinklist);
    }
  }
}
